import "Gillothrin.Items.Utils";
import "Gillothrin.Items.GUI";
import "Gillothrin.Items.TooltipWindow";

ItemStat = class( Turbine.UI.Control );
function ItemStat:Constructor(item,name,textWidth,parent,charWidthInPixels, optionsObj)
  Turbine.UI.Control.Constructor(self);
  if(item ~= nil) then
    self.item = item;
    self:SetParent(parent);
    local labelWidth = 0;
    if(textWidth == -1) then
      labelWidth = string.len(item:GetQuantity()) * charWidthInPixels;
    else
      labelWidth = textWidth * charWidthInPixels;
    end

    local topPadding = optionsObj.topPadding;
    local bottomPadding = optionsObj.bottomPadding;
    local leftPadding = optionsObj.leftPadding;
    local spacer = optionsObj.spacer;
    local imgWidth = optionsObj.iconImageWidth;
    local fgHeight = optionsObj.iconImageHeight;
    
    local backgroundHeight = fgHeight + topPadding+ bottomPadding;
    local textStart = imgWidth+leftPadding+spacer;
    
    self:SetSize(labelWidth+leftPadding+textStart,backgroundHeight);
    self:SetItemStatBackgroundColor(optionsObj.frameback);

    local imageBackground = Turbine.UI.Control();
    self.imageBackground = imageBackground;
    
    imageBackground:SetParent(self);
    self:SetIconbackgroundColor(optionsObj.iconback);
    imageBackground:SetPosition(leftPadding, topPadding);
    imageBackground:SetSize(imgWidth, fgHeight);
    imageBackground:SetMouseVisible(false);
   
    local itemImage = Turbine.UI.Control();        
    itemImage:SetParent(imageBackground);  
    itemImage:SetPosition(0,0); 
    itemImage:SetBackground(item:GetSmallImage());
    itemImage:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    itemImage:SetSize(imgWidth, fgHeight);
    itemImage.name = name;
    self.itemImage = itemImage;
    itemImage:SetMouseVisible(false);
    
    local itemText = VLabel(item:GetQuantity(),textStart,topPadding,labelWidth,fgHeight);
    itemText:SetParent(self);
    itemText:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
    self.itemText = itemText;
    self:SetItemTextBackgroundColor(optionsObj.nrback);
    self:SetItemTextForeGroundColor(optionsObj.textcolor);
    itemText:SetMouseVisible(false);
    

    local this = self;
    self.ItemQuantityChanged = function(s,a)
      local originalText = this.itemText:GetText();
      local newText = this.item:GetQuantity();
      if(string.len(originalText) ~= string.len(newText)) then
        _G.eventWalletChange(this,this.item);
      else
        this.itemText:SetText(newText);
      end
    end
    self.item:Listen(self.ItemQuantityChanged);

    self.mouseButton = nil;

    self.MouseUpHandler = function(s,a)
      parent.mouseButton, parent.mousePressedX, parent.mousePressedY = nil,nil,nil;
    end;
    self.MouseDownHandler = function(s,a)
      parent.mouseButton = a.Button;
      parent.mousePressedX = a.X;
      parent.mousePressedY = a.Y;
    end;

    self.MouseMoveHandler = function(s,a)
      if(parent.mouseButton == Turbine.UI.MouseButton.Left) then
        local left, top = parent:GetPosition();
        local width, height = parent:GetSize();
        local x = left + (a.X - parent.mousePressedX);
        local y = top + (a.Y - parent.mousePressedY);
        local x,y = ValidatePosition(x,y,width,height);
        parent:SetPosition(x,y);
        parent.optionsObj.mainLeft = x;
        parent.optionsObj.mainTop = y;
      end
    end;

    AddCallback(self,"MouseUp",self.MouseUpHandler);
    AddCallback(self,"MouseDown",self.MouseDownHandler);
    AddCallback(self,"MouseMove",self.MouseMoveHandler);
    
    self.tooltip = TooltipWindow(self,self.itemImage.name);
  end
end

function ItemStat:SetIconbackgroundColor(iconback)
  self.imageBackground:SetBackColor(Turbine.UI.Color(rgb(iconback.a),rgb(iconback.r),rgb(iconback.g),rgb(iconback.b)));
end

function ItemStat:SetItemStatBackgroundColor(frameback)
  self:SetBackColor(Turbine.UI.Color(rgb(frameback.a),rgb(frameback.r),rgb(frameback.g),rgb(frameback.b)));
end

function ItemStat:SetItemTextBackgroundColor(nrback)
  self.itemText:SetBackColor(Turbine.UI.Color(rgb(nrback.a),rgb(nrback.r),rgb(nrback.g),rgb(nrback.b)));
end

function ItemStat:SetItemTextForeGroundColor(fcolor)
  self.itemText:SetForeColor(Turbine.UI.Color(rgb(fcolor.a),rgb(fcolor.r),rgb(fcolor.g),rgb(fcolor.b)));
end

function ItemStat:Cleanup()
  self.item:Unhook();
  self:SetVisible(false);
  self.tooltip:SetParent(nil);
  self.tooltip = nil;
  self:SetParent(nil);
  RemoveCallback(self,"MouseUp",self.MouseUpHandler);
  RemoveCallback(self,"MouseDown",self.MouseDownHandler);
  RemoveCallback(self,"MouseMove",self.MouseMoveHandler);
end
