import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Gillothrin.Items.GUI";
import "Gillothrin.Items.WalletOptionItem";
import "Gillothrin.Items.Utils";
import "Gillothrin.Items.SelectionPane";
import "Gillothrin.Items.BackGroundSelectorWindow";

OptionsControl = class(Turbine.UI.Control);
function OptionsControl:Constructor(optionsObj,wallet)
  Turbine.UI.Control.Constructor(self);

  self.selectionListBox = Turbine.UI.ListBox();
  self.selectionListBox:SetParent(self);

  self.selectionPane = SelectionPane(optionsObj,wallet);
  self.selectionPane:SetParent(self);
  self:SetBackColor(Turbine.UI.Color(0,0,0));
  self.optionsObj = optionsObj;
  self.selectionListBox:AddItem(self.selectionPane);

  self.textcolorSelector = BackGroundSelectorWindow(L["textcolor"],optionsObj.textcolor);
  self.textcolorSelector:SetParent(self);
  self.textcolorChangeHandler = function(s,a) ExecuteCallback(self,"ColorChanged",a); end;
  AddCallback(self.textcolorSelector,"ColorChanged",self.textcolorChangeHandler);
  self.selectionListBox:AddItem(self.textcolorSelector);

  self.framebackgroundSelector = BackGroundSelectorWindow(L["framebackground"],optionsObj.frameback);
  self.framebackgroundSelector:SetParent(self);
  self.framebackgroundcolorChangeHandler = function(s,a)  ExecuteCallback(self,"ColorChanged",a); end;
  AddCallback(self.framebackgroundSelector,"ColorChanged",self.framebackgroundcolorChangeHandler);
  self.selectionListBox:AddItem(self.framebackgroundSelector);

  self.backgroundSelector = BackGroundSelectorWindow(L["iconbackground"],optionsObj.iconback);
  self.backgroundSelector:SetParent(self);
  self.backgroundcolorChangeHandler = function(s,a) ExecuteCallback(self,"ColorChanged",a); end;
  AddCallback(self.backgroundSelector,"ColorChanged",self.backgroundcolorChangeHandler);
  self.selectionListBox:AddItem(self.backgroundSelector);

  self.nrbackgroundSelector = BackGroundSelectorWindow(L["numberbackground"],optionsObj.nrback);
  self.nrbackgroundSelector:SetParent(self);
  self.nrbackgroundcolorChangeHandler = function(s,a) ExecuteCallback(self,"ColorChanged",a); end;
  AddCallback(self.nrbackgroundSelector,"ColorChanged",self.nrbackgroundcolorChangeHandler);
  self.selectionListBox:AddItem(self.nrbackgroundSelector);

  self.vertbarSelector = BackGroundSelectorWindow(L["vertbarbackground"],optionsObj.vertback);
  self.vertbarSelector:SetParent(self);
  self.vertbarcolorChangeHandler = function(s,a) ExecuteCallback(self,"BarColorChanged",a); end;
  AddCallback(self.vertbarSelector,"BarColorChanged",self.vertbarcolorChangeHandler);
  self.selectionListBox:AddItem(self.vertbarSelector);

  self.checkers = Turbine.UI.Control();
  self.checkers:SetParent(self);

  self.forceOneLine = Turbine.UI.Control();
  self.forceOneLine:SetParent(self.checkers);
  self.forceOneLine:SetPosition(0,0);
  self.forceOneLine:SetHeight(self.optionsObj.defaultRowHeight);

  self.forceOneLine.checkBox = VCheckBox(self.optionsObj.oneline,0,0,100,self.optionsObj.defaultRowHeight);
  self.forceOneLine.checkBox:SetParent(self.forceOneLine);
  self.forceOneLine.checkBox:SetText(L["forceoneline"]);

  self.forceNoPack = Turbine.UI.Control();
  self.forceNoPack:SetParent(self.checkers);
  self.forceNoPack:SetPosition(0,self.optionsObj.defaultRowHeight);
  self.forceNoPack:SetHeight(self.optionsObj.defaultRowHeight);

  self.forceNoPack.checkBox = VCheckBox(self.optionsObj.nopack,0,0,100,self.optionsObj.defaultRowHeight);
  self.forceNoPack.checkBox:SetParent(self.forceNoPack);
  self.forceNoPack.checkBox:SetText(L["forcenopack"]);

  local this = self;
  AddCallback(self.forceOneLine.checkBox,"CheckedChanged",function(s,a)
    this.optionsObj.oneline = s:IsChecked();
    _G.eventWalletChange(s,a);
  end);

  AddCallback(self.forceNoPack.checkBox, "CheckedChanged",function(s,a)
    this.optionsObj.nopack = s:IsChecked();
    _G.eventWalletChange(s,a);
  end);

  self.checkers:SetHeight(self.forceOneLine:GetHeight()+self.forceNoPack:GetHeight());
  self.selectionListBox:AddItem(self.checkers);

  self.vbar = Turbine.UI.Lotro.ScrollBar();
  self.vbar:SetParent(self);
  self.vbar:SetOrientation(Turbine.UI.Orientation.Vertical);

  self.selectionListBox:SetVerticalScrollBar(self.vbar);
  
end

function OptionsControl:RepaintOptionsControl()
  self.selectionPane:Layout();
  local width, height = self.selectionPane:GetSize();

  self.textcolorSelector:SetWidth(width);
  self.textcolorSelector:Layout();

  height = height + self.textcolorSelector:GetHeight();

  self.framebackgroundSelector:SetWidth(width);
  self.framebackgroundSelector:Layout();

  height = height + self.framebackgroundSelector:GetHeight();

  self.backgroundSelector:SetWidth(width);
  self.backgroundSelector:Layout();

  height = height + self.backgroundSelector:GetHeight();

  self.nrbackgroundSelector:SetWidth(width);
  self.nrbackgroundSelector:Layout();

  height = height + self.nrbackgroundSelector:GetHeight();

  self.vertbarSelector:SetWidth(width);
  self.vertbarSelector:Layout();

  height = height + self.vertbarSelector:GetHeight();

  self.checkers:SetWidth(width);
  self.forceNoPack:SetWidth(width);
  self.forceNoPack.checkBox:SetWidth(width);
  self.forceOneLine:SetWidth(width);
  self.forceOneLine.checkBox:SetWidth(width);

  height = height + self.checkers:GetHeight();

  self.selectionListBox:SetSize(width,550);
  self:SetSize(width,550+2);
  self.vbar:SetPosition(width-10,0);
  self.vbar:SetSize(10,self:GetHeight());
end

function OptionsControl:ShowOptionsControl()
  self:RepaintOptionsControl();
  self:SetVisible(true);
end

function OptionsControl:Unload()
  self:SetVisible(false);
  self.textcolorSelector:Unload();
  self.framebackgroundSelector:Unload();
  self.backgroundSelector:Unload();
  self.nrbackgroundSelector:Unload();
  self.vertbarSelector:Unload();

  RemoveCallback(self.textcolorSelector,"ColorChanged",self.textcolorChangeHandler);
  RemoveCallback(self.framebackgroundSelector,"ColorChanged",self.framebackgroundcolorChangeHandler);
  RemoveCallback(self.backgroundSelector,"ColorChanged",self.backgroundcolorChangeHandler);
  RemoveCallback(self.nrbackgroundSelector,"ColorChanged",self.nrbackgroundcolorChangeHandler);
  RemoveCallback(self.vertbarSelector,"BarColorChanged",self.vertbarcolorChangeHandler);
  self.selectionPane:Unload();
end


