import "Turbine";
import "Turbine.Gameplay";
import "Gillothrin.Items.TabbedPane";
import "Gillothrin.Items.DataLoader";
import "Gillothrin.Items.MainOptionsControl";
import "Gillothrin.Items.OptionsWindow";
import "Gillothrin.Items.ItemWalletInstance";

ItemsPlugin = class(Turbine.Object);
function ItemsPlugin:Constructor()
  Turbine.Object.Constructor(self);

  self.loader = DataLoader();
  self.itemRemovedPlaceHolder = {};
  self.windows = {};
  self.optionsWindow = OptionsWindow(self);
  self.tempOptionsPanel = Turbine.UI.Control();
  self.tempOptionsPanel:SetSize(200,200);
  self.tempOptionsPanel.launch = VButton(L["launchoptions"],75,100,150,20);
  self.tempOptionsPanel.launch:SetParent(self.tempOptionsPanel);
  self.tempOptionsPanel.launch.MouseClick = function(s,a)
    self.optionsWindow:Layout();
    self.optionsWindow:SetVisible(true);
    self.optionsWindow:Activate();
  end;

  local player = Turbine.Gameplay.LocalPlayer:GetInstance();

  for i=1,#self.loader.storage do
    local wstorage = self.loader.storage[i];
    local w = ItemWalletInstance(wstorage,player);
    self.optionsWindow:AddControl(wstorage.tabname,w.optionsControl);
    table.insert(self.windows,w);
  end

  AddCallback(self,"InstanceAdded", function(s,a)
    local s = DataOptions();
    s.tabname = a.name;
    self.loader:Add(s);
    local w = ItemWalletInstance(s,player);
    self.optionsWindow:AddControl(s.tabname,w.optionsControl);
    table.insert(self.windows,w);
    self.optionsWindow:Layout();    
  end);

  AddCallback(self,"InstanceRemoved",function(s,a)
    local instance = self.windows[a.index-1];
    self.optionsWindow:RemoveControl(a.index);
    instance:Unload();
    table.remove(self.windows,a.index-1);
    self.loader:Remove(a.index-1);
  end);
  
  AddCallback(player:GetWallet(), "ItemAdded", eventWalletItemAdded);
  AddCallback(player:GetWallet(), "ItemRemoved", eventWalletItemRemoved);
  self.tabnamehandler = function(s,a) 
    local sint = self.loader.storage[a.idx-1];
    sint.tabname = a.tabname;
  end
  AddCallback(self.optionsWindow,"TabNameChanged",self.tabnamehandler);
  AddCallback(self.optionsWindow,"SelectionTabChanged",function(s,a) 
    local instance = self.windows[a.idx-1];
    
  end);
  self.optionsWindow:Layout();
end

function ItemsPlugin:SetVisible(flag)
  for i=1,#self.windows do
    if flag then
      self.windows[i]:Refresh();
    end
    self.windows[i]:SetVisible(flag);
  end
end

function ItemsPlugin:Update()
  for i=1,#self.windows do
    self.windows[i]:Update();
  end
end

function ItemsPlugin:Refresh()
  for i=1,#self.windows do
    self.windows[i]:Refresh();
  end
end

function ItemsPlugin:ShowOptionsControl()
  self:GetOptionsPanel();
  self.optionsWindow:Layout();
  self.optionsWindow:SetVisible(true);
end

function ItemsPlugin:ResetWindows()
  for i=1,#self.windows do
    self.windows[i]:SetDefaultPosition(i);
  end
end

function ItemsPlugin:Unload()
  local player = Turbine.Gameplay.LocalPlayer:GetInstance();
  RemoveCallback(player:GetWallet(), "ItemAdded", eventWalletItemAdded);
  RemoveCallback(player:GetWallet(), "ItemRemoved", eventWalletItemRemoved);
  RemoveCallback(Turbine.Chat,"Received",chatItemRemovedEvent);

  _G.eventWalletChange = nil;
  _G.eventWalletItemRemoved = nil;
  _G.chatItemRemovedEvent = nil;
  _G.eventWalletItemAdded = nil;

  if Turbine.Shell.IsCommand(shellCommandName) then
    Turbine.Shell.RemoveCommand(shellCommandName);
  end

  self.loader:SaveData();

  for i=1,#self.windows do
    self.windows[i]:Unload();
    self.windows[i] = nil;
  end
  self.mainOptionsTabbedPane = nil;

end

function ItemsPlugin:GetOptionsPanel()
  return self.tempOptionsPanel;
end

function ItemsPlugin:RegisterShellCommand()
  myCommandLine = Turbine.ShellCommand();
  function myCommandLine:Execute( name, shellCommand )
    shellCommand = string.lower(shellCommand);
    if (shellCommand == "setup") or (shellCommand == "options") then
      itemsPlugin:ShowOptionsControl();
    elseif (shellCommand == "show") or (shellCommand == "on") then
      itemsPlugin:SetVisible(true);
    elseif (shellCommand == "hide") or (shellCommand == "off") then
      itemsPlugin:SetVisible(false);
    elseif shellCommand == "refresh" then
      eventWalletItemAdded();
    elseif shellCommand == "reset" then
      itemsPlugin:ResetWindows();
    else
      myCommandLine:GetHelp();
    end
  end

  function myCommandLine:GetShortHelp()
    echo("Command: <rgb=#FFFFFF>/"..shellCommandName.."</rgb> [show|on] [hide|off] [setup|options] [refresh] [reset] [help]");
  end

  function myCommandLine:GetHelp()
    echo(string.rep("=",string.len(pluginFullIdText)));
    echo(pluginFullIdText);
    echo(string.rep("=",string.len(pluginFullIdText)));
    echo("USAGE: /"..shellCommandName.." [command]\n");
    echo("COMMANDS:");
    echo("help - show this help (also if no command)");
    echo("show|on - show item tracking window");
    echo("hide|off - hide item tracking window");
    echo("setup|options - show the options window");
    echo("refresh - refresh your tracked items");
    echo("reset - reset the window to default");
    echo("");
  end
  return myCommandLine;
end

_G.eventWalletItemAdded = function (s,a)
  itemsPlugin:Refresh();
end

_G.eventWalletChange = function(sender, args)
  itemsPlugin:Update();
end

--[[
  Ugly hack because the Wallet.ItemRemoved event is triggering before the item is actually 
  removed from the wallet making a refresh then ineffective. It should work like the item 
  triggers an QuantityChange event and THEN the wallet triggers an ItemRemoved event 
  (when the item is gone from the wallet).
]]--  
_G.eventWalletItemRemoved = function(s,a)

  local obj = Turbine.Object();
  if a.Item:GetQuantity() > 1 then
    obj.items = a.Item:GetQuantity();
  end
  obj.name = a.Item:GetName();
  table.insert(itemsPlugin.itemRemovedPlaceHolder,obj);

  AddCallback(Turbine.Chat,"Received",chatItemRemovedEvent);

end

_G.chatItemRemovedEvent = function(s,a)
  if a.ChatType == Turbine.ChatType.SelfLoot then
    local pkeys = {};
    for k,v in pairs(itemsPlugin.itemRemovedPlaceHolder) do
      table.insert(pkeys,k);
    end
    local m = false;
    local rmsg = L["ItemRemoved"];
    for _,k in ipairs(pkeys) do
      if a.Message:sub(0,string.len(rmsg)) == rmsg then
        local name = GetMessageData(a.Message);
        if name == itemsPlugin.itemRemovedPlaceHolder[k].name then
          itemsPlugin.itemRemovedPlaceHolder[k] = nil;
          break;
        end
      end
    end
    local cnt = 0;
    for k,v in pairs(itemsPlugin.itemRemovedPlaceHolder) do cnt = cnt + 1; end
    if cnt == 0 then
      itemsPlugin:Update();
      RemoveCallback(Turbine.Chat,"Received",chatItemRemovedEvent);
    end
  end
end
