import "Turbine.UI";
import "Gillothrin.Items.Tab";

TabbedPane = class(Turbine.UI.Control);
function TabbedPane:Constructor()
    Turbine.UI.Control.Constructor(self);

    self.tabs = {};
    self.selected = nil;
    self:SetMouseVisible(false);
    
    -- border
    self.border = Turbine.UI.Control();
    self.border:SetParent(self);
    self.border:SetZOrder(-1);
    self.border:SetPosition(1, 24);
    self.border:SetMouseVisible(false);
    local blueBorderColor = Turbine.UI.Color(0.15, 0.2, 0.3);  
    self.border:SetBackColor(blueBorderColor);
end

function TabbedPane:FindTabIndex(tab)
    for i=1, #self.tabs do
      if tab == self.tabs[i] then return i end;
    end
    return -1;
end

function TabbedPane:AddTab(name, content)
    local tab = Tab(name, content);
    tab.handler = function(s,a)
      local idx = self:FindTabIndex(a.tab);
      ExecuteCallback(self,"TabNameChanged",{idx=idx,tabname=a.tabname});
    end;
    
    AddCallback(tab,"NameChanged",tab.handler);
    
    tab.Click = function(sender, args)
        local idx = self:FindTabIndex(tab);
        if idx > 0 then self:SelectTab(idx); end;
    end
    table.insert(self.tabs, tab);
    
    tab:SetParent(self);
    content:SetParent(self);
    content:SetVisible(false);
    return tab;
end

function TabbedPane:RemoveTab(index)
  local tab = self.tabs[index];
  if self.selected == tab then
    self.selected:SetSelected(false);
    self.selected:GetContent():SetVisible(false);
    if #self.tabs > 0 then
      self.selected = self.tabs[1];
      self.selected:SetSelected(true);
      self.selected:GetContent():SetVisible(true);
      self:FireEvent();
    end
  end
  RemoveCallback(tab,"NameChanged",tab.handler);
  tab.content:SetParent(nil);
  tab:SetParent(nil);
  tab.content = nil;
  table.remove(self.tabs,index);
  self:Layout();
end

function TabbedPane:SelectTab(index)
    if (self.selected == self.tabs[index]) then
        return;
    end
    
    if (self.selected ~= nil) then
        self.selected:SetSelected(false);
        self.selected:GetContent():SetVisible(false);
    end
    
    self.selected = self.tabs[index];
    self.selected:SetSelected(true);
    self.selected:GetContent():SetVisible(true);
    self:FireEvent();
    self:Layout();
end

function TabbedPane:GetTabs()
    return self.tabs;
end

function TabbedPane:GetSelectedTab()
    return self.selected;
end

function TabbedPane:FireEvent()
    ExecuteCallback(self, "SelectedTabChanged", {selection=self.selected, idx = self:FindTabIndex(self.selected)});
end

function TabbedPane:Layout()
    local width, height = self:GetSize();
    local x,y = self.selected:GetContent():GetSize();
    self.border:SetSize(x+3,y+26);
    self:SetHeight(y+26);
    
    local tabWidth = 0;
    for i = 1, table.getn(self.tabs) do
        local tab = self.tabs[i];
        tab:SetPosition(tabWidth, 2);
        tabWidth = tabWidth + tab:GetWidth();
    end
    if tabWidth > width then self:SetWidth(tabWidth); end;
    if (self.selected ~= nil) then
        local content = self.selected:GetContent();
        content:SetPosition(2, 25);
    end
end
